@@ -10,24 +10,27 @@ from sql import Column, Null, Window, Literal
 from sql.aggregate import Sum, Max
 from sql.conditionals import Coalesce, Case
 
-from trytond.model import ModelView, ModelSQL, fields, Unique, sequence_ordered
+from trytond.model import (
+    ModelView, ModelSQL, fields, Unique, sequence_ordered, tree)
 from trytond.wizard import Wizard, StateView, StateAction, StateTransition, \
     Button
 from trytond.report import Report
 from trytond.tools import reduce_ids, grouped_slice
-from trytond.pyson import Eval, If, PYSONEncoder
+from trytond.pyson import Eval, If, PYSONEncoder, Bool
 from trytond.transaction import Transaction
 from trytond.pool import Pool
-from trytond import backend
 
-__all__ = ['TypeTemplate', 'Type',
+from .common import PeriodMixin, ActivePeriodMixin
+
+__all__ = ['TypeTemplate', 'Type', 'OpenType',
     'AccountTemplate', 'AccountTemplateTaxTemplate',
     'Account', 'AccountDeferral', 'AccountTax',
     'OpenChartAccountStart', 'OpenChartAccount',
     'GeneralLedgerAccount', 'GeneralLedgerAccountContext',
     'GeneralLedgerLine', 'GeneralLedgerLineContext',
     'GeneralLedger', 'TrialBalance',
-    'BalanceSheetContext', 'IncomeStatementContext',
+    'BalanceSheetContext', 'BalanceSheetComparisionContext',
+    'IncomeStatementContext',
     'AgedBalanceContext', 'AgedBalance', 'AgedBalanceReport',
     'CreateChartStart', 'CreateChartAccount', 'CreateChartProperties',
     'CreateChart', 'UpdateChartStart', 'UpdateChartSucceed', 'UpdateChart']
@@ -41,7 +44,8 @@ def inactive_records(func):
     return wrapper
 
 
-class TypeTemplate(sequence_ordered(), ModelSQL, ModelView):
+class TypeTemplate(
+        sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
     'Account Type Template'
     __name__ = 'account.account.type.template'
     name = fields.Char('Name', required=True)
@@ -56,21 +60,6 @@ class TypeTemplate(sequence_ordered(), ModelSQL, ModelView):
         ('credit-debit', 'Credit - Debit'),
         ], 'Display Balance', required=True)
 
-    @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        table = TableHandler(cls, module_name)
-
-        super(TypeTemplate, cls).__register__(module_name)
-
-        # Migration from 2.4: drop required on sequence
-        table.not_null_action('sequence', action='remove')
-
-    @classmethod
-    def validate(cls, records):
-        super(TypeTemplate, cls).validate(records)
-        cls.check_recursion(records, rec_name='name')
-
     @staticmethod
     def default_balance_sheet():
         return False
@@ -83,12 +72,6 @@ class TypeTemplate(sequence_ordered(), ModelSQL, ModelView):
     def default_display_balance():
         return 'debit-credit'
 
-    def get_rec_name(self, name):
-        if self.parent:
-            return self.parent.get_rec_name(name) + '\\' + self.name
-        else:
-            return self.name
-
     def _get_type_value(self, type=None):
         '''
         Set the values for account creation.
@@ -146,18 +129,26 @@ class TypeTemplate(sequence_ordered(), ModelSQL, ModelView):
             childs = sum((c.childs for c in childs), ())
 
 
-class Type(sequence_ordered(), ModelSQL, ModelView):
+class Type(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
     'Account Type'
     __name__ = 'account.account.type'
-    name = fields.Char('Name', size=None, required=True)
+
+    _states = {
+        'readonly': (Bool(Eval('template', -1)) &
+            ~Eval('template_override', False)),
+        }
+    name = fields.Char('Name', size=None, required=True, states=_states)
     parent = fields.Many2One('account.account.type', 'Parent',
-        ondelete="RESTRICT", domain=[
+        ondelete="RESTRICT", states=_states,
+        domain=[
             ('company', '=', Eval('company')),
-            ], depends=['company'])
+            ],
+        depends=['company'])
     childs = fields.One2Many('account.account.type', 'parent', 'Children',
         domain=[
             ('company', '=', Eval('company')),
-        ], depends=['company'])
+        ],
+        depends=['company'])
     currency_digits = fields.Function(fields.Integer('Currency Digits'),
             'get_currency_digits')
     amount = fields.Function(fields.Numeric('Amount',
@@ -166,30 +157,22 @@ class Type(sequence_ordered(), ModelSQL, ModelView):
     amount_cmp = fields.Function(fields.Numeric('Amount',
         digits=(16, Eval('currency_digits', 2)), depends=['currency_digits']),
         'get_amount_cmp')
-    balance_sheet = fields.Boolean('Balance Sheet')
-    income_statement = fields.Boolean('Income Statement')
+    balance_sheet = fields.Boolean('Balance Sheet', states=_states)
+    income_statement = fields.Boolean('Income Statement', states=_states)
     display_balance = fields.Selection([
         ('debit-credit', 'Debit - Credit'),
         ('credit-debit', 'Credit - Debit'),
-        ], 'Display Balance', required=True)
+        ], 'Display Balance', required=True, states=_states)
     company = fields.Many2One('company.company', 'Company', required=True,
             ondelete="RESTRICT")
     template = fields.Many2One('account.account.type.template', 'Template')
-
-    @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        table = TableHandler(cls, module_name)
-
-        super(Type, cls).__register__(module_name)
-
-        # Migration from 2.4: drop required on sequence
-        table.not_null_action('sequence', action='remove')
-
-    @classmethod
-    def validate(cls, types):
-        super(Type, cls).validate(types)
-        cls.check_recursion(types, rec_name='name')
+    template_override = fields.Boolean('Override Template',
+        help="Check to override template definition",
+        states={
+            'invisible': ~Bool(Eval('template', -1)),
+            },
+        depends=['template'])
+    del _states
 
     @staticmethod
     def default_balance_sheet():
@@ -203,6 +186,10 @@ class Type(sequence_ordered(), ModelSQL, ModelView):
     def default_display_balance():
         return 'debit-credit'
 
+    @classmethod
+    def default_template_override(cls):
+        return False
+
     def get_currency_digits(self, name):
         return self.company.currency.digits
 
@@ -242,7 +229,8 @@ class Type(sequence_ordered(), ModelSQL, ModelView):
                     ])
             for child in childs:
                 res[type_.id] += type_sum[child.id]
-            res[type_.id] = type_.company.currency.round(res[type_.id])
+            exp = Decimal(str(10.0 ** -type_.currency_digits))
+            res[type_.id] = res[type_.id].quantize(exp)
             if type_.display_balance == 'credit-debit':
                 res[type_.id] = - res[type_.id]
         return res
@@ -254,7 +242,7 @@ class Type(sequence_ordered(), ModelSQL, ModelView):
         if not current.get('comparison'):
             return dict.fromkeys([t.id for t in types], None)
         new = {}
-        for key, value in current.iteritems():
+        for key, value in current.items():
             if key.endswith('_cmp'):
                 new[key[:-4]] = value
         with transaction.set_context(new):
@@ -267,12 +255,6 @@ class Type(sequence_ordered(), ModelSQL, ModelView):
                 ~Eval('comparison', False)),
             ]
 
-    def get_rec_name(self, name):
-        if self.parent:
-            return self.parent.get_rec_name(name) + '\\' + self.name
-        else:
-            return self.name
-
     @classmethod
     def delete(cls, types):
         types = cls.search([
@@ -294,7 +276,7 @@ class Type(sequence_ordered(), ModelSQL, ModelView):
         childs = [self]
         while childs:
             for child in childs:
-                if child.template:
+                if child.template and not child.template_override:
                     vals = child.template._get_type_value(type=child)
                     if vals:
                         values.append([child])
@@ -305,7 +287,47 @@ class Type(sequence_ordered(), ModelSQL, ModelView):
             self.write(*values)
 
 
-class AccountTemplate(ModelSQL, ModelView):
+class OpenType(Wizard):
+    'Open Type'
+    __name__ = 'account.account.open_type'
+    start = StateTransition()
+    account = StateAction('account.act_account_balance_sheet')
+    ledger_account = StateAction('account.act_account_general_ledger')
+
+    def transition_start(self):
+        context_model = Transaction().context.get('context_model')
+        if context_model == 'account.balance_sheet.comparision.context':
+            return 'account'
+        elif context_model == 'account.income_statement.context':
+            return 'ledger_account'
+        return 'end'
+
+    def open_action(self, action):
+        pool = Pool()
+        Type = pool.get('account.account.type')
+        type = Type(Transaction().context['active_id'])
+        action['name'] = '%s (%s)' % (action['name'], type.rec_name)
+        trans_context = Transaction().context
+        context = {
+            'active_id': trans_context.get('active_id'),
+            'active_ids': trans_context.get('active_ids', []),
+            'active_model': trans_context.get('active_model'),
+            }
+        context_model = trans_context.get('context_model')
+        if context_model:
+            Model = pool.get(context_model)
+            for fname in Model._fields.keys():
+                if fname == 'id':
+                    continue
+                context[fname] = trans_context.get(fname)
+        action['pyson_context'] = PYSONEncoder().encode(context)
+        return action, {}
+
+    do_account = open_action
+    do_ledger_account = open_action
+
+
+class AccountTemplate(PeriodMixin, tree(), ModelSQL, ModelView):
     'Account Template'
     __name__ = 'account.account.template'
     name = fields.Char('Name', size=None, required=True, select=True)
@@ -336,10 +358,16 @@ class AccountTemplate(ModelSQL, ModelView):
             'invisible': Eval('kind') == 'view',
             }, depends=['kind'])
     party_required = fields.Boolean('Party Required',
+        domain=[
+            If((Eval('kind') == 'view') | ~Eval('deferral', False),
+                ('party_required', '=', False),
+                (),
+                )
+            ],
         states={
-            'invisible': Eval('kind') == 'view',
+            'invisible': (Eval('kind') == 'view') | ~Eval('deferral', False),
             },
-        depends=['kind'])
+        depends=['kind', 'deferral'])
     general_ledger_balance = fields.Boolean('General Ledger Balance',
         states={
             'invisible': Eval('kind') == 'view',
@@ -349,6 +377,11 @@ class AccountTemplate(ModelSQL, ModelView):
     taxes = fields.Many2Many('account.account.template-account.tax.template',
             'account', 'tax', 'Default Taxes',
             domain=[('parent', '=', None)])
+    replaced_by = fields.Many2One(
+        'account.account.template', "Replaced By",
+        states={
+            'invisible': ~Eval('end_date'),
+            })
 
     @classmethod
     def __setup__(cls):
@@ -356,11 +389,6 @@ class AccountTemplate(ModelSQL, ModelView):
         cls._order.insert(0, ('code', 'ASC'))
         cls._order.insert(1, ('name', 'ASC'))
 
-    @classmethod
-    def validate(cls, templates):
-        super(AccountTemplate, cls).validate(templates)
-        cls.check_recursion(templates)
-
     @staticmethod
     def default_kind():
         return 'view'
@@ -409,6 +437,10 @@ class AccountTemplate(ModelSQL, ModelView):
             res['code'] = self.code
         if not account or account.kind != self.kind:
             res['kind'] = self.kind
+        if not account or account.start_date != self.start_date:
+            res['start_date'] = self.start_date
+        if not account or account.end_date != self.end_date:
+            res['end_date'] = self.end_date
         if not account or account.reconcile != self.reconcile:
             res['reconcile'] = self.reconcile
         if not account or account.deferral != self.deferral:
@@ -470,10 +502,10 @@ class AccountTemplate(ModelSQL, ModelView):
             create(childs)
             childs = sum((c.childs for c in childs), ())
 
-    def update_account_taxes(self, template2account, template2tax,
+    def update_account2(self, template2account, template2tax,
             template_done=None):
         '''
-        Update recursively account taxes based on template.
+        Update recursively account taxes and replaced_by based on template.
         template2account is a dictionary with template id as key and account id
         as value, used to convert template id into account.
         template2tax is a dictionary with tax template id as key and tax id as
@@ -494,15 +526,20 @@ class AccountTemplate(ModelSQL, ModelView):
             to_write = []
             for template in templates:
                 if template.id not in template_done:
+                    template_done.append(template.id)
+                    account = Account(template2account[template.id])
+                    if account.template_override:
+                        continue
+                    values = {}
                     if template.taxes:
                         tax_ids = [template2tax[x.id] for x in template.taxes]
-                        to_write.append(
-                            [Account(template2account[template.id])])
-                        to_write.append({
-                                'taxes': [
-                                    ('add', tax_ids)],
-                                })
-                    template_done.append(template.id)
+                        values['taxes'] = [('add', tax_ids)]
+                    if template.replaced_by:
+                        values['replaced_by'] = template2account[
+                            template.replaced_by.id]
+                    if values:
+                        to_write.append([account])
+                        to_write.append(values)
             if to_write:
                 Account.write(*to_write)
 
@@ -522,12 +559,15 @@ class AccountTemplateTaxTemplate(ModelSQL):
             ondelete='RESTRICT', select=True, required=True)
 
 
-class Account(ModelSQL, ModelView):
+class Account(ActivePeriodMixin, tree(), ModelSQL, ModelView):
     'Account'
     __name__ = 'account.account'
-    name = fields.Char('Name', size=None, required=True, select=True)
-    code = fields.Char('Code', size=None, select=True)
-    active = fields.Boolean('Active', select=True)
+    _states = {
+        'readonly': (Bool(Eval('template', -1)) &
+            ~Eval('template_override', False)),
+        }
+    name = fields.Char('Name', required=True, select=True, states=_states)
+    code = fields.Char('Code', select=True, states=_states)
     company = fields.Many2One('company.company', 'Company', required=True,
             ondelete="RESTRICT")
     currency = fields.Function(fields.Many2One('currency.currency',
@@ -541,6 +581,7 @@ class Account(ModelSQL, ModelView):
             ('id', '!=', Eval('currency', -1)),
             ],
         states={
+            'reaodnly': _states['readonly'],
             'invisible': (Eval('kind').in_(
                     ['payable', 'revenue', 'receivable', 'expense'])
                 | ~Eval('deferral', False)),
@@ -552,6 +593,7 @@ class Account(ModelSQL, ModelView):
         states={
             'invisible': Eval('kind') == 'view',
             'required': Eval('kind') != 'view',
+            'readonly': _states['readonly'],
             },
         domain=[
             ('company', '=', Eval('company')),
@@ -559,10 +601,13 @@ class Account(ModelSQL, ModelView):
     parent = fields.Many2One('account.account', 'Parent', select=True,
         left="left", right="right", ondelete="RESTRICT",
         domain=[('company', '=', Eval('company'))],
-        depends=['company'])
+        states=_states, depends=['company'])
     left = fields.Integer('Left', required=True, select=True)
     right = fields.Integer('Right', required=True, select=True)
-    childs = fields.One2Many('account.account', 'parent', 'Children')
+    childs = fields.One2Many(
+        'account.account', 'parent', 'Children',
+        domain=[('company', '=', Eval('company'))],
+        depends=['company'])
     balance = fields.Function(fields.Numeric('Balance',
         digits=(16, Eval('currency_digits', 2)), depends=['currency_digits']),
         'get_balance')
@@ -584,6 +629,7 @@ class Account(ModelSQL, ModelView):
         help='Allow move lines of this account \nto be reconciled.',
         states={
             'invisible': Eval('kind') == 'view',
+            'readonly': _states['readonly'],
             }, depends=['kind'])
     note = fields.Text('Note')
     kind = fields.Selection([
@@ -594,19 +640,27 @@ class Account(ModelSQL, ModelView):
             ('expense', 'Expense'),
             ('stock', 'Stock'),
             ('view', 'View'),
-            ], 'Kind', required=True)
+            ], 'Kind', required=True, states=_states)
     deferral = fields.Boolean('Deferral', states={
             'invisible': Eval('kind') == 'view',
+            'readonly': _states['readonly'],
             }, depends=['kind'])
     deferrals = fields.One2Many('account.account.deferral', 'account',
         'Deferrals', readonly=True, states={
             'invisible': Eval('kind') == 'view',
             }, depends=['kind'])
     party_required = fields.Boolean('Party Required',
+        domain=[
+            If((Eval('kind') == 'view') | ~Eval('deferral', False),
+                ('party_required', '=', False),
+                (),
+                )
+            ],
         states={
-            'invisible': Eval('kind') == 'view',
+            'invisible': (Eval('kind') == 'view') | ~Eval('deferral', False),
+            'readonly': _states['readonly'],
             },
-        depends=['kind'])
+        depends=['kind', 'deferral'])
     general_ledger_balance = fields.Boolean('General Ledger Balance',
         states={
             'invisible': Eval('kind') == 'view',
@@ -622,11 +676,31 @@ class Account(ModelSQL, ModelView):
             help=('Default tax for manual encoding of move lines \n'
                 'for journal types: "expense" and "revenue"'),
             depends=['company'])
+    replaced_by = fields.Many2One(
+        'account.account', "Replaced By",
+        domain=[('company', '=', Eval('company', -1))],
+        states={
+            'readonly': _states['readonly'],
+            'invisible': ~Eval('end_date'),
+            },
+        depends=['company'])
     template = fields.Many2One('account.account.template', 'Template')
+    template_override = fields.Boolean('Override Template',
+        help="Check to override template definition",
+        states={
+            'invisible': ~Bool(Eval('template', -1)),
+            },
+        depends=['template'])
+    del _states
 
     @classmethod
     def __setup__(cls):
         super(Account, cls).__setup__()
+        for date in [cls.start_date, cls.end_date]:
+            date.states = {
+                'readonly': (Bool(Eval('template', -1))
+                    & ~Eval('template_override', False)),
+                }
         cls._error_messages.update({
                 'delete_account_containing_move_lines': ('You can not delete '
                     'account "%s" because it has move lines.'),
@@ -652,7 +726,6 @@ class Account(ModelSQL, ModelView):
     @classmethod
     def validate(cls, accounts):
         super(Account, cls).validate(accounts)
-        cls.check_recursion(accounts)
         cls.check_second_currency(accounts)
 
     @staticmethod
@@ -663,10 +736,6 @@ class Account(ModelSQL, ModelView):
     def default_right():
         return 0
 
-    @staticmethod
-    def default_active():
-        return True
-
     @staticmethod
     def default_company():
         return Transaction().context.get('company') or None
@@ -691,6 +760,10 @@ class Account(ModelSQL, ModelView):
     def default_kind():
         return 'view'
 
+    @classmethod
+    def default_template_override(cls):
+        return False
+
     def get_currency(self, name):
         return self.company.currency.id
 
@@ -714,7 +787,7 @@ class Account(ModelSQL, ModelView):
         table_c = cls.__table__()
         line = MoveLine.__table__()
         ids = [a.id for a in accounts]
-        balances = dict((i, 0) for i in ids)
+        balances = dict((i, Decimal(0)) for i in ids)
         line_query, fiscalyear_ids = MoveLine.query_get(line)
         for sub_ids in grouped_slice(ids):
             red_sql = reduce_ids(table_a.id, sub_ids)
@@ -725,20 +798,17 @@ class Account(ModelSQL, ModelView):
                     ).select(
                     table_a.id,
                     Sum(Coalesce(line.debit, 0) - Coalesce(line.credit, 0)),
-                    where=red_sql & line_query & (table_c.active == True),
+                    where=red_sql & line_query,
                     group_by=table_a.id))
             result = cursor.fetchall()
             balances.update(dict(result))
 
-        # SQLite uses float for SUM
-        for account_id, balance in balances.iteritems():
-            if isinstance(balance, Decimal):
-                break
-            balances[account_id] = Decimal(str(balance))
-
         for account in accounts:
-            balances[account.id] = account.company.currency.round(
-                balances[account.id])
+            # SQLite uses float for SUM
+            if not isinstance(balances[account.id], Decimal):
+                balances[account.id] = Decimal(str(balances[account.id]))
+            exp = Decimal(str(10.0 ** -account.currency_digits))
+            balances[account.id] = balances[account.id].quantize(exp)
 
         fiscalyears = FiscalYear.browse(fiscalyear_ids)
 
@@ -764,7 +834,7 @@ class Account(ModelSQL, ModelView):
         for name in names:
             if name not in {'credit', 'debit', 'amount_second_currency'}:
                 raise ValueError('Unknown name: %s' % name)
-            result[name] = dict((i, 0) for i in ids)
+            result[name] = dict((i, Decimal(0)) for i in ids)
 
         table = cls.__table__()
         line = MoveLine.__table__()
@@ -790,12 +860,11 @@ class Account(ModelSQL, ModelView):
         for account in accounts:
             for name in names:
                 if name == 'amount_second_currency':
-                    currency = account.second_currency
+                    exp = Decimal(str(10.0 ** -account.second_currency_digits))
                 else:
-                    currency = account.company.currency
-                if currency:
-                    result[name][account.id] = currency.round(
-                        result[name][account.id])
+                    exp = Decimal(str(10.0 ** -account.currency_digits))
+                result[name][account.id] = (
+                    result[name][account.id].quantize(exp))
 
         cumulate_names = []
         if Transaction().context.get('cumulate'):
@@ -927,31 +996,12 @@ class Account(ModelSQL, ModelView):
             default = {}
         else:
             default = default.copy()
-        default.setdefault('template')
+        default.setdefault('template', None)
         default.setdefault('deferrals', [])
         new_accounts = super(Account, cls).copy(accounts, default=default)
         cls._rebuild_tree('parent', None, 0)
         return new_accounts
 
-    @classmethod
-    def write(cls, *args):
-        pool = Pool()
-        MoveLine = pool.get('account.move.line')
-        actions = iter(args)
-        args = []
-        for accounts, values in zip(actions, actions):
-            if not values.get('active', True):
-                childs = cls.search([
-                        ('parent', 'child_of', [a.id for a in accounts]),
-                        ])
-                if MoveLine.search([
-                            ('account', 'in', [a.id for a in childs]),
-                            ]):
-                    values = values.copy()
-                    del values['active']
-            args.extend((accounts, values))
-        super(Account, cls).write(*args)
-
     @classmethod
     def delete(cls, accounts):
         MoveLine = Pool().get('account.move.line')
@@ -985,7 +1035,7 @@ class Account(ModelSQL, ModelView):
         childs = [self]
         while childs:
             for child in childs:
-                if child.template:
+                if child.template and not child.template_override:
                     vals = child.template._get_account_value(account=child)
                     current_type = child.type.id if child.type else None
                     if child.template.type:
@@ -1003,9 +1053,9 @@ class Account(ModelSQL, ModelView):
         if values:
             self.write(*values)
 
-    def update_account_taxes(self, template2account, template2tax):
+    def update_account2(self, template2account, template2tax):
         '''
-        Update recursively account taxes base on template.
+        Update recursively account taxes and replaced_by base on template.
         template2account is a dictionary with template id as key and account id
         as value, used to convert template id into account.
         template2tax is a dictionary with tax template id as key and tax id as
@@ -1017,7 +1067,7 @@ class Account(ModelSQL, ModelView):
         if template2tax is None:
             template2tax = {}
 
-        values = []
+        to_write = []
         childs = [self]
         while childs:
             for child in childs:
@@ -1025,22 +1075,49 @@ class Account(ModelSQL, ModelView):
                     continue
                 if not child.template.taxes:
                     continue
+                values = {}
                 tax_ids = [template2tax[x.id] for x in child.template.taxes
                     if x.id in template2tax]
                 old_tax_ids = [x.id for x in child.taxes]
                 for tax_id in tax_ids:
                     if tax_id not in old_tax_ids:
-                        values.append([child])
-                        values.append({
-                                'taxes': [
-                                    ('add', template2tax[x.id])
-                                    for x in self.template.taxes
-                                    if x.id in template2tax],
-                                })
+                        values['taxes'] = [
+                            ('add', template2tax[x.id])
+                            for x in self.template.taxes
+                            if x.id in template2tax]
                         break
+                if child.template.replaced_by:
+                    replaced_by = template2account[
+                        child.template.replaced_by.id]
+                else:
+                    replaced_by = None
+                old_replaced_by = (
+                    child.replaced_by.id if child.replaced_by else None)
+                if old_replaced_by != replaced_by:
+                    values['replaced_by'] = replaced_by
+                if values:
+                    to_write.append([child])
+                    to_write.append(values)
             childs = sum((c.childs for c in childs), ())
-        if values:
-            self.write(*values)
+        if to_write:
+            self.write(*to_write)
+
+    def current(self, date=None):
+        "Return the actual account for the date"
+        pool = Pool()
+        Date = pool.get('ir.date')
+        context = Transaction().context
+        if date is None:
+            date = context.get('date') or Date.today()
+        if self.start_date and date < self.start_date:
+            return None
+        elif self.end_date and self.end_date < date:
+            if self.replaced_by:
+                return self.replaced_by.current(date=date)
+            else:
+                return None
+        else:
+            return self
 
 
 class AccountDeferral(ModelSQL, ModelView):
@@ -1176,15 +1253,15 @@ class OpenChartAccount(Wizard):
         return 'end'
 
 
-class GeneralLedgerAccount(ModelSQL, ModelView):
+class GeneralLedgerAccount(ActivePeriodMixin, ModelSQL, ModelView):
     'General Ledger Account'
     __name__ = 'account.general_ledger.account'
 
     # TODO reuse rec_name of Account
     name = fields.Char('Name')
     code = fields.Char('Code')
-    active = fields.Boolean('Active')
     company = fields.Many2One('company.company', 'Company')
+    type = fields.Many2One('account.account.type', 'Type')
     start_debit = fields.Function(fields.Numeric('Start Debit',
             digits=(16, Eval('currency_digits', 2)),
             depends=['currency_digits']),
@@ -1236,7 +1313,7 @@ class GeneralLedgerAccount(ModelSQL, ModelView):
         Account = pool.get('account.account')
         account = Account.__table__()
         columns = []
-        for fname, field in cls._fields.iteritems():
+        for fname, field in cls._fields.items():
             if not hasattr(field, 'set'):
                 columns.append(Column(account, fname).as_(fname))
         return account.select(*columns,
@@ -1508,7 +1585,7 @@ class GeneralLedgerLine(ModelSQL, ModelView):
         move = Move.__table__()
         account = Account.__table__()
         columns = []
-        for fname, field in cls._fields.iteritems():
+        for fname, field in cls._fields.items():
             if hasattr(field, 'set'):
                 continue
             field_line = getattr(Line, fname, None)
@@ -1632,17 +1709,11 @@ class BalanceSheetContext(ModelView):
     date = fields.Date('Date', required=True)
     company = fields.Many2One('company.company', 'Company', required=True)
     posted = fields.Boolean('Posted Move', help='Show only posted move')
-    comparison = fields.Boolean('Comparison')
-    date_cmp = fields.Date('Date', states={
-            'required': Eval('comparison', False),
-            'invisible': ~Eval('comparison', False),
-            },
-        depends=['comparison'])
 
     @staticmethod
     def default_date():
         Date_ = Pool().get('ir.date')
-        return Date_.today()
+        return Transaction().context.get('date', Date_.today())
 
     @staticmethod
     def default_company():
@@ -1650,7 +1721,18 @@ class BalanceSheetContext(ModelView):
 
     @staticmethod
     def default_posted():
-        return False
+        return Transaction().context.get('posted', False)
+
+
+class BalanceSheetComparisionContext(BalanceSheetContext):
+    'Balance Sheet Context'
+    __name__ = 'account.balance_sheet.comparision.context'
+    comparison = fields.Boolean('Comparison')
+    date_cmp = fields.Date('Date', states={
+            'required': Eval('comparison', False),
+            'invisible': ~Eval('comparison', False),
+            },
+        depends=['comparison'])
 
     @classmethod
     def default_comparison(cls):
@@ -1914,9 +1996,9 @@ class AgedBalance(ModelSQL, ModelView):
         # Ensure None are before 0 to get the next index pointing to the next
         # value and not a None value
         term_values = sorted(
-            terms.values(), key=lambda x: ((x is not None), x or 0))
+            list(terms.values()), key=lambda x: ((x is not None), x or 0))
 
-        for name, value in terms.iteritems():
+        for name, value in terms.items():
             if value is None or factor is None or date is None:
                 columns.append(Literal(None).as_(name))
                 continue
@@ -1936,7 +2018,6 @@ class AgedBalance(ModelSQL, ModelView):
                 condition=reconciliation.id == line.reconciliation
             ).select(*columns,
                 where=(line.party != Null)
-                & (account.active == True)
                 & account.kind.in_(kind)
                 & ((line.reconciliation == Null)
                     | (reconciliation.date > date))
@@ -2070,19 +2151,39 @@ class CreateChart(Wizard):
             ])
     create_properties = StateTransition()
 
+    @classmethod
+    def __setup__(cls):
+        super(CreateChart, cls).__setup__()
+        cls._error_messages.update({
+                'account_chart_exists': ('A chart of accounts already exists '
+                    'for the company "%(company)s".')
+                })
+
     def transition_create_account(self):
         pool = Pool()
         TaxCodeTemplate = pool.get('account.tax.code.template')
+        TaxCodeLineTemplate = pool.get('account.tax.code.line.template')
         TaxTemplate = pool.get('account.tax.template')
         TaxRuleTemplate = pool.get('account.tax.rule.template')
         TaxRuleLineTemplate = \
             pool.get('account.tax.rule.line.template')
         Config = pool.get('ir.configuration')
+        Account = pool.get('account.account')
+        transaction = Transaction()
 
-        with Transaction().set_context(language=Config.get_language(),
-                company=self.account.company.id):
+        company = self.account.company
+        # Skip access rule
+        with transaction.set_user(0):
+            accounts = Account.search([('company', '=', company.id)], limit=1)
+        if accounts:
+            self.raise_user_warning('duplicated_chart.%d' % company.id,
+                'account_chart_exists', {
+                    'company': company.rec_name,
+                    })
+
+        with transaction.set_context(language=Config.get_language(),
+                company=company.id):
             account_template = self.account.account_template
-            company = self.account.company
 
             # Create account types
             template2type = {}
@@ -2097,23 +2198,29 @@ class CreateChart(Wizard):
                 template2account=template2account,
                 template2type=template2type)
 
+            # Create taxes
+            template2tax = {}
+            TaxTemplate.create_tax(
+                account_template.id, company.id,
+                template2account=template2account,
+                template2tax=template2tax)
+
             # Create tax codes
             template2tax_code = {}
             TaxCodeTemplate.create_tax_code(
                 account_template.id, company.id,
                 template2tax_code=template2tax_code)
 
-            # Create taxes
-            template2tax = {}
-            TaxTemplate.create_tax(
-                account_template.id, company.id,
+            # Create tax code lines
+            template2tax_code_line = {}
+            TaxCodeLineTemplate.create_tax_code_line(
+                account_template.id,
+                template2tax=template2tax,
                 template2tax_code=template2tax_code,
-                template2account=template2account,
-                template2tax=template2tax)
+                template2tax_code_line=template2tax_code_line)
 
-            # Update taxes on accounts
-            account_template.update_account_taxes(template2account,
-                template2tax)
+            # Update taxes and replaced_by on accounts
+            account_template.update_account2(template2account, template2tax)
 
             # Create tax rules
             template2rule = {}
@@ -2178,6 +2285,8 @@ class UpdateChart(Wizard):
         pool = Pool()
         TaxCode = pool.get('account.tax.code')
         TaxCodeTemplate = pool.get('account.tax.code.template')
+        TaxCodeLine = pool.get('account.tax.code.line')
+        TaxCodeLineTemplate = pool.get('account.tax.code.line.template')
         Tax = pool.get('account.tax')
         TaxTemplate = pool.get('account.tax.template')
         TaxRule = pool.get('account.tax.rule')
@@ -2209,6 +2318,19 @@ class UpdateChart(Wizard):
                 template2account=template2account,
                 template2type=template2type)
 
+        # Update taxes
+        template2tax = {}
+        Tax.update_tax(
+            company.id,
+            template2account=template2account,
+            template2tax=template2tax)
+        # Create missing taxes
+        if account.template:
+            TaxTemplate.create_tax(
+                account.template.id, account.company.id,
+                template2account=template2account,
+                template2tax=template2tax)
+
         # Update tax codes
         template2tax_code = {}
         TaxCode.update_tax_code(
@@ -2220,23 +2342,23 @@ class UpdateChart(Wizard):
                 account.template.id, company.id,
                 template2tax_code=template2tax_code)
 
-        # Update taxes
-        template2tax = {}
-        Tax.update_tax(
+        # Update tax code lines
+        template2tax_code_line = {}
+        TaxCodeLine.update_tax_code_line(
             company.id,
+            template2tax=template2tax,
             template2tax_code=template2tax_code,
-            template2account=template2account,
-            template2tax=template2tax)
-        # Create missing taxes
+            template2tax_code_line=template2tax_code_line)
+        # Create missing tax code lines
         if account.template:
-            TaxTemplate.create_tax(
-                account.template.id, account.company.id,
+            TaxCodeLineTemplate.create_tax_code_line(
+                account.template.id,
+                template2tax=template2tax,
                 template2tax_code=template2tax_code,
-                template2account=template2account,
-                template2tax=template2tax)
+                template2tax_code_line=template2tax_code_line)
 
-        # Update taxes on accounts
-        account.update_account_taxes(template2account, template2tax)
+        # Update taxes and replaced_by on accounts
+        account.update_account2(template2account, template2tax)
 
         # Update tax rules
         template2rule = {}
